// Boid.h
// Boid structure definition and convenience function for binary serialisation-
// and deserialisation
//
// The MIT License (MIT)
//
// Copyright (c) 2014 de.uni-hamburg.informatik.wr.papo
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef SWARMFLOCKING_SIMULATION_BOID_H
#define SWARMFLOCKING_SIMULATION_BOID_H

#include "Vector.h"
#include <fstream>

struct Boid {
    Vector3f position;
    Vector3f velocity;
    Vector3f force;

    // Write this structure to disk, as is.
    // This only works if the endianess does not change!
    inline void serialize(std::ostream &out, bool positionOnly = false)
    {
        position.serialize(out);

        if (!positionOnly) {
            // omit padding field
            velocity.serialize(out);
        }
    }

    // Read a Boid from disk and store it in the current object.
    // This only works if the endianess does not change!
    inline void deserialize(std::istream &in, bool positionOnly = false)
    {
        position.deserialize(in);

        if (positionOnly) {
            velocity = { 0.0f, 0.0f, 0.0f }; // Default value for velocity
        } else {
            velocity.deserialize(in);
        }
    }
};

typedef Boid Predator;

#endif /* SWARMFLOCKING_SIMULATION_BOID_H */
