#include "PartialForce.h"

#include "../common/Flocking.h"
#include <cassert>

PartialForce::PartialForce() :
    vec{0.0f, 0.0f, 0.0f}, influenceCount{0}
{
}

PartialForce::~PartialForce()
{
}

PartialForce::PartialForce(const PartialForce &pf) : 
    vec{pf.vec}, influenceCount{pf.influenceCount}
{
}

PartialForce::PartialForce(PartialForce &&pf) :
    vec{std::move(pf.vec)}, influenceCount{std::move(pf.influenceCount)}
{
}

PartialForce::PartialForce(const Vector3f &vec, const unsigned int influenceCount) :
    vec{vec}, influenceCount{influenceCount}
{
}

PartialForce& PartialForce::operator=(const PartialForce& rhs)
{
    vec = rhs.vec;
    influenceCount = rhs.influenceCount;

    return *this;
}

PartialForce& PartialForce::operator=(PartialForce &&rhs)
{
    vec = std::move(rhs.vec);
    influenceCount = std::move(rhs.influenceCount);

    return *this;
}

PartialForce PartialForce::operator+(const PartialForce &pf) const
{
    return { vec + pf.vec, influenceCount + pf.influenceCount };
}

PartialForce PartialForce::operator+(const Vector3f &v) const
{
    return { vec + v, influenceCount + 1 };
}

PartialForce& PartialForce::operator+=(const PartialForce &pf)
{
    vec += pf.vec;
    influenceCount += pf.influenceCount;

    return *this;
}

PartialForce& PartialForce::operator+=(const Vector3f &v)
{
    vec += v;
    ++influenceCount;

    return *this;
}

/*
    This routine calls the correct finalize* method, that can be
    found in src/common/Flocking.h, depending on the supplied arguments
    Actor (describing the current object - either Boid or Predator) and
    the kind of force that is being computed
*/
Vector3f PartialForce::finalize(Boid &b, Actor r, Force kind)
{
    Vector3f ret;

    switch(kind) {
        case Force::Alignment:
            ret = finalizeAlignment(b, vec, influenceCount, 
                                    (r == Actor::Boid) ? configuration.boid.weight.alignment 
                                                : configuration.predator.weight.alignment);
            break;
        case Force::Cohesion:
            ret = finalizeCohesion(b, vec, influenceCount, 
                                   (r == Actor::Boid) ? configuration.boid.weight.cohesion 
                                               : configuration.predator.weight.cohesion);
            break;
        case Force::Separation:
            ret = finalizeSeparation(b, vec, influenceCount, 
                                     (r == Actor::Boid) ? configuration.boid.weight.separation 
                                                 : configuration.predator.weight.separation);
            break;
    }
    
    return ret;
}
