// PartialForce.h
// Used as a performance optimization, so that all processes calculate part of
// the force, then combine them to compute the (close to) correct result.
// Since floating point math is neither associative nor commutative, the result
// will be slighly off.
//
// The MIT License (MIT)
//
// Copyright (c) 2014 de.uni-hamburg.informatik.wr.papo
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef SWARMFLOCKING_SIMULATION_PARTIALFORCE_H
#define SWARMFLOCKING_SIMULATION_PARTIALFORCE_H

#include "../common/Vector.h"
#include "../common/Boid.h"

#pragma pack(1)
struct PartialForce
{
    PartialForce();
    ~PartialForce();
    PartialForce(const PartialForce &pf);
    PartialForce(const Vector3f &vec, const unsigned int influenceCount);
    PartialForce(PartialForce &&pf);

    PartialForce& operator=(const PartialForce& rhs);
    PartialForce& operator=(PartialForce &&rhs);

    PartialForce operator+(const PartialForce &pf) const;
    PartialForce operator+(const Vector3f &v) const;

    PartialForce& operator+=(const PartialForce &pf);
    PartialForce& operator+=(const Vector3f &v);

    enum class Actor {
        Boid,
        Predator
    };

    enum class Force {
        Alignment,
        Cohesion,
        Separation
    };

    Vector3f finalize(Boid &b, Actor r, Force kind);

    Vector3f vec;
    unsigned int influenceCount;
};
#endif /* SWARMFLOCKING_SIMULATION_PARTIALFORCE_H */
