// BoidBuffer.h
// Handles deserialisation of boid data from input streams.
//
// The MIT License (MIT)
//
// Copyright (c) 2014 de.uni-hamburg.informatik.wr.papo
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef SWARMFLOCKING_VISUALISATION_BOIDBUFFER_H
#define SWARMFLOCKING_VISUALISATION_BOIDBUFFER_H

#include "../common/Boid.h"
#include "../common/SwarmStorageConfiguration.h"

#include <fstream>
#include <memory> // for unique_pt

class BoidBuffer
{
public:
    BoidBuffer(std::istream &input, const SwarmStorageConfiguration &configuration);
    ~BoidBuffer();

    struct SwarmBuf {
        std::size_t nboids;
        std::size_t npredators;

        Boid * boids;
        Boid * predators;

        bool isValid;
    };

    SwarmBuf next(); // Switch this out for std::unique_ptr
private:
    SwarmBuf requestNext(Boid *, Boid *);
    Boid     *boidA, *boidB; // storage for boids
    Predator *predA, *predB; // storage for predators

    std::size_t boidCount;
    std::size_t predatorCount;

    SwarmStorageConfiguration config;
    bool bufferEmpty;
    std::istream &input;
};


#endif /* SWARMFLOCKING_VISUALISATION_BOIDBUFFER_H */
