#include "../common/Vector.h"
#include "../common/Boid.h"
#include "../common/Configuration.h"
#include "../common/Flocking.h"

#include <mutex> // call_once
#include <ctime> // time
#include <cstdlib> // srand, rand
#include <vector>
#include <cassert>
#include <functional>

#define sq(x) ((x) * (x))

void kmeans(Boid *boids, std::size_t nboids, std::vector<int> &cluster_out,
            const std::size_t k /* number of cluster */,
            const std::size_t c /* number of iterations */)
{
    assert((nboids == cluster_out.size()) && "Invalid parameters");

    auto randomVec = []() -> Vector3f {
        if (configuration.deterministic) {
            static std::once_flag once;
            std::call_once(once, []() {
                std::srand(std::time(nullptr));
            });
        }

        return { static_cast<float>(rand() % static_cast<int>(configuration.worldSize)),
                 static_cast<float>(rand() % static_cast<int>(configuration.worldSize)),
                 static_cast<float>(rand() % static_cast<int>(configuration.worldSize)) };
    };

    std::vector<Vector3f> centroids(k);

    // random initialisation of centroids
    for (Vector3f &vec : centroids) {
        vec = randomVec();
    }

    for (std::size_t i = 0; i < c; ++i) {
        for (std::size_t m = 0; m < nboids; ++m) {
            Boid &b = boids[m];
            int correspondingCluster = -1;
            // sufficiently large starting value
            float minimumLength = sqrt(sq(configuration.worldSize) +
                                       sq(configuration.worldSize) +
                                       sq(configuration.worldSize));

            // find nearest centroid
            for (std::size_t j = 0; j < centroids.size(); ++j) {
                float l = calculateDifference(b.position, centroids[j]).length();
                if (l < minimumLength) {
                    minimumLength = l;
                    correspondingCluster = j;
                }
            }

            // nearest centroid
            cluster_out[m] = correspondingCluster;
        }

        // update centroids
        for (std::size_t j = 0; j < centroids.size(); ++j) {
            Vector3f sum{};
            std::size_t influences = 0;

            for (std::size_t m = 0; m < cluster_out.size(); ++m) {
                if (cluster_out[m] == j) {
                    influences++;
                    sum += boids[m].position;
                }
            }

            if (influences > 0) {
                sum /= static_cast<float>(influences);
                centroids[j] = sum;
            }
        }
    }

}
