// SwarmStorageConfiguration.h
// Defines settings to allow serialisation and deserialisation to work correctly
//
// The MIT License (MIT)
//
// Copyright (c) 2014 de.uni-hamburg.informatik.wr.papo
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef SWARMFLOCKING_VISUALISATION_SWARMSTORAGECONFIGURATION_H
#define SWARMFLOCKING_VISUALISATION_SWARMSTORAGECONFIGURATION_H

#include <fstream>

#pragma pack(1)
struct SwarmStorageConfiguration {
    // Number of boids in each frame.
    std::size_t boidCount;

    // Number of predators in each frame.
    std::size_t predatorCount; 

    // Offset from beginning of frame where boids are stored.
    std::size_t boidOff; 

    // Offset from beginning of frame where predators are stored.
    std::size_t predatorOff; 

    // Flag to determine iff only the position of each boid is stored.
    bool positionOnly;
    
    /*
        Write this configuration to file.
    */
    inline void serialize(std::ostream &to)
    {
        to.write(reinterpret_cast<char *>(this), sizeof(SwarmStorageConfiguration));
    }

    /*
        Read the configuration from a file.
    */
    inline void deserialize(std::istream &from)
    {
        from.read(reinterpret_cast<char *>(this), sizeof(SwarmStorageConfiguration));
    }
};

#endif /* SWARMFLOCKING_VISUALISATION_SWARMSTORAGECONFIGURATION_H */