// Visualisation.h
// Manages visual presentation of data using SFML. 
//
// The MIT License (MIT)
//
// Copyright (c) 2014 de.uni-hamburg.informatik.wr.papo
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef SWARMFLOCKING_VISUALISATION_VISUALISATION_H
#define SWARMFLOCKING_VISUALISATION_VISUALISATION_H

#include <SFML/Graphics.hpp>
#include <fstream>
#include "BoidBuffer.h"

class Visualisation {
public:
    Visualisation(sf::String title, // window title
                  std::istream &input_source, // input source for boid data
                  bool singleStepping, // overwrites fps, visualisation now gets controlled by the 'space' key.
                  std::size_t fps); // number of fps to be rendered

    void run();

    void handleSizeEvent(sf::Event::SizeEvent sizeEvent);
    void handleKeyEvent(sf::Event::KeyEvent keyEvent);
    void handleTextEvent(sf::Event::TextEvent textEvent);
    void handleMouseMoveEvent(sf::Event::MouseMoveEvent mouseMoveEvent);
    void handleMouseButtonEvent(sf::Event::MouseButtonEvent mouseButtonEvent);
    void handleMouseWheelEvent(sf::Event::MouseWheelEvent mouseWheelEvent);

private:
    void renderCube();
    void renderSwarm(BoidBuffer::SwarmBuf &buffer);

    sf::RenderWindow window;
    sf::Font font;

    bool singleStepping; // a special mode where the user can request the next frame to be rendered
    bool nextFrameRequested; // set to true iff the next frame should be rendered (only used in in single stepping mode!)
    bool running;

    float rotationSpeed = 2.0f;
    float rotation[2] = {
        20.0f,
        0.0f
    };

    BoidBuffer dataSource;
};

#endif /* SWARMFLOCKING_VISUALISATION_VISUALISATION_H */